/* dcwebui.js */
"use strict";

var $ = (document.querySelectorAll ? 
	function(s) {
		var r = document.querySelectorAll(s);
		return (r.length == 1) ? r[0] : r;
	} : 
	function(s) {
		// i'm not writing a selector engine today... i'm *not*...
		if (! s.length) return [];
		if (s[0] == '#') {
			return document.getElementById(s.slice(1));
		} else if (s[0] == '.') {
			return document.getElementsByClassName(s.slice(1))
		} else {
			return document.getElementsByTagName(s);
		}
	}
);

var hesc = function(s) {
	var filter = {
		'&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&#34;', '\'': '&#39;'
	};
	return s.toString().replace(/[&<>'"]/g, function(s) { return filter[s]; });
}

var write = {
	'cls': function() {
		$('#mainchat').innerHTML = '';
		return this;
	},
	'scroll': function() {
		var e = $('#mainchat');
		e.scrollTop = e.scrollHeight;
		return this;
	},
	'raw': function(s) {
		$('#mainchat').innerHTML += s;
		return this.scroll();
	},
	'c': function(c, s) {
		return this.raw('<span class="'+c+'">'+s+'</span>');
	},
	'time': function() {
		var d = new Date();
		var pad = function(s) { return s < 10 ? '0'+s : ''+s ; };
		return this.raw(
			'<span class="tx-time">['+
			pad(d.getHours()) + ":" + pad(d.getMinutes())+
			"]</span> "
		);
	},
	'system': function(s) {
		return this.time().c('tx-sys', hesc(s)).raw('<br/>');
	},
	'public': function(u, s) {
		return this.time().
			c('tx-user', '&lt;'+hesc(u)+'&gt;').raw('&nbsp;').
			c('tx-chat', hesc(s)).raw('<br/>');
	},
	'private': function(f, t, s) {
		return this.time().
			c('tx-user', '&lt;'+hesc(f)+'&nbsp;-&gt;&nbsp'+hesc(t)+'&gt;').
			raw('&nbsp').
			c('tx-private', hesc(s)).
			raw('<br/>');
	}
};

var userlist = {
	'add': function(u) {
		var to_add = document.createElement('li');
		to_add.innerHTML = hesc(u);
		to_add.onclick = start_pm;
		
		var users = $('#userlist').children;
		var cmp   = hesc(u).toUpperCase();
		for (var i = 0; i < users.length; i++) {
			if ((''+users[i].innerHTML).toUpperCase() > cmp) {
				$('#userlist').insertBefore(to_add, users[i]);
				return this;
			}
		}
		$('#userlist').appendChild(to_add);
		return this;
	},
	'del': function(u) {
		var users = $('#userlist').children;
		var cmp   = hesc(u).toUpperCase();
		for (var i = 0; i < users.length; i++) {
			if ((''+users[i].innerHTML).toUpperCase() == cmp) {
				$('#userlist').removeChild(users[i]);
				break;
			}
		}
		return this;
	},
	'clear': function() {
		var users = $('#userlist').children;
		for (var i = 0; i < users.length; i++) {
			$('#userlist').removeChild(users[i]);
		}
		return this;
	},
	'count': function() {
		return $("#userlist").children.length;
	}
};

var start_pm = function(e) {
	$("#chatbox").value = "/pm "+e.srcElement.innerText+" ";
	toggle_pages();
};

var submit = function() {
	var str = $("#chatbox").value;
	if (! str.length) return;

	if (hub_state == 0) {
		hub_state = 1;
		hub_last_nick = str;
		sock.emit('hello', {'nick' : hub_last_nick, 'pass' : ''});
		$("#chatbox").disabled = true;
		write.system("Connecting...");
		
	} else if (hub_state == 2) {
		if (str.length > 3 && str.slice(0,3).toLowerCase() == '/pm') {
			var parts = str.split(' ');
			var send = {'user': parts[1], 'message': parts.slice(2).join(' ')};
			sock.emit('priv', send);
			write.private( hub_last_nick, send.user, send.message );
		} else {
			sock.emit('pub', {'message' : str});
		}
		
	} else {
		write.system("Invalid internal state.");
	}
	
	$("#chatbox").value = '';
};

var toggle_pages = function() {
	var toggle = function(el) {
		el.style.display=(el.style.display=='block')?'none':'block';
	}
	toggle($("#tab-main"));
	toggle($("#tab-users"));
	write.scroll();
};

var sock = {};
var hub_state     = 0; // [disconnected, sent-nick, connected]
var hub_last_nick = '';
var hub_hubname   = 'DCWebUI';

window.onload = function() {
	write.system("Communicating with server...");
	
	$("title").innerText = DCWEBUI_CONF.title;
	
	// HTML event handlers
	
	$("#button-userlist").onclick = toggle_pages;
	
	$("#form-none").onsubmit = function() {
		submit();
		return false; // don't submit form
	};
	
	// Socket event handlers
	
	sock = io.connect('http://'+DCWEBUI_CONF.extern);
	sock.on('cls', function() {
		write.cls();
		hub_state = 0;
	});
	sock.on('hubname', function(s) {
		write.system("Now talking on "+s);
		hub_hubname = s;
		$("title").innerText = "("+userlist.count()+") "+hub_hubname;
	});
	sock.on('sys', function(data) {
		write.system(data);
	});
	sock.on('raw', function(data) {
		write.raw(data);
	});
	sock.on('pub', function(data) {
		write['public']( data.user, data.message);
	});
	sock.on('priv', function(data) {
		write['private']( data.user, hub_last_nick, data.message);
	});
	sock.on('hello', function() {
		hub_state = 2;
		$("#chatbox").disabled = false;
	});
	sock.on('part', function(u) {
		userlist.del(u.user);
		$("title").innerText = "("+userlist.count()+") "+hub_hubname;
	});
	sock.on('join', function(u) {
		userlist.add(u.user);
		$("title").innerText = "("+userlist.count()+") "+hub_hubname;
	});
	sock.on('close', function() {
		hub_state = 0;
		userlist.clear();
		write.system("Connection closed by remote host.");
	});
	sock.on('disconnect', function() {
		hub_state = 0;
		userlist.clear();
		write.system("Lost connection to the server.");
	});
};
