/* dcwebui.js */
"use strict";

var $ = (document.querySelectorAll ? 
	function(s) {
		var r = document.querySelectorAll(s);
		return (s[0] == '#' && r.length == 1) ? r[0] : r;
	} : 
	function(s) {
		// i'm not writing a selector engine today... i'm *not*...
		if (! s.length) return [];
		if (s[0] == '#') {
			return document.getElementById(s.slice(1));
		} else if (s[0] == '.') {
			return document.getElementsByClassName(s.slice(1))
		} else {
			return document.getElementsByTagName(s);
		}
	}
);

var hesc = function(s) {
	var filter = {
		'&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&#34;', '\'': '&#39;'
	};
	return s.toString().replace(/[&<>'"]/g, function(s) { return filter[s]; });
}

var nl2br = function(str) { // thanks php.js!
	return (str+'').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1<br>$2');
}

var linkify = function(str) {
	return str.replace(
		/(https?:\/\/[^\s<]+)/g, "<a target='_blank' href=\"$1\">$1</a>"
	);
}

var sanitise = function(s) {
	return linkify(nl2br(hesc(s)));
}

var write = function(tab) {
	var $tab = $('#inner-'+tab);
	return {
		'cls': function() {
			$tab.innerHTML = '';
			return this;
		},
		'scroll': function() {
			$tab.scrollTop = $tab.scrollHeight;
			return this;
		},
		'raw': function(s) {
			$tab.innerHTML += s;
			return this.scroll();
		},
		'c': function(c, s) {
			return this.raw('<span class="'+c+'">'+s+'</span>');
		},
		'time': function() {
			var d = new Date();
			var pad = function(s) {
				return (s < 10) ? '0'+s : ''+s ;
			};
			return this.raw(
				'<span class="tx-time">['+
				pad(d.getHours()) + ":" + pad(d.getMinutes())+
				"]</span> "
			);
		},
		'system': function(s) {
			return this.time().c('tx-sys', sanitise(s)).raw('<br/>');
		},
		'pub': function(u, s) {
			return this.time().
				c('tx-user', '&lt;'+hesc(u)+'&gt;').raw('&nbsp;').
				c('tx-chat', sanitise(s)).raw('<br/>');
		},
		'priv': function(f, t, s) {
			return this.time().
				c('tx-user', '&lt;'+hesc(f)+'&nbsp;-&gt;&nbsp'+hesc(t)+'&gt;').
				raw('&nbsp').
				c('tx-private', sanitise(s)).
				raw('<br/>');
		}
	};
}

var userlist = {
	'add': function(u) {
		var userlists = $(".userlist");
		for (var i in userlists) {
			if (! userlists[i].children) continue;
			var userlist = userlists[i];
			
			var to_add = document.createElement('li');
			to_add.innerHTML = hesc(u);
			to_add.onclick = function() {
				writerFor(u);          // create
				tab_set(pm_tabs[u]);   // switch
				writerFor(u).scroll(); // scroll
			};
			
			var users = userlist.children;
			var cmp   = hesc(u).toUpperCase();
			for (var i = 0; i < users.length; i++) {
				if ((''+users[i].innerHTML).toUpperCase() > cmp) {
					userlist.insertBefore(to_add, users[i]);
					return this;
				}
			}
			userlist.appendChild(to_add);
		}
		return this;
	},
	'del': function(u) {
		var userlists = $(".userlist");
		for (var i in userlists) {
			if (! userlists[i].children) continue;
			var userlist = userlists[i];
				
			var users = userlist.children;
			var cmp   = hesc(u).toUpperCase();
			for (var i = 0; i < users.length; i++) {
				if ((''+users[i].innerHTML).toUpperCase() == cmp) {
					userlist.removeChild(users[i]);
					break;
				}
			}
		}
		return this;
	},
	'clear': function() {
		var userlists = $(".userlist");
		for (var i in userlists) {
			if (! userlists[i].children) continue;
			var userlist = userlists[i];
			
			var users = userlist.children;
			for (var j = 0; j < users.length; j++) {
				userlist.removeChild(users[j]);
			}
		}
		return this;
	},
	'count': function() {
		return $(".userlist")[0].children.length;
	}
};

var start_pm = function(e) {
	$("#chatbox").value = "/pm "+e.srcElement.innerText+" ";
	tab_set('tab-users');
};

var submit = function() {
	var str = $("#chatbox").value;
	if (! str.length) return;

	if (hub_state == 0) {
		hub_state = 1;
		
		var name_parts = str.split(":", 2);		
		hub_last_nick = name_parts[0];
				
		sock.emit('hello', {'nick' : hub_last_nick, 'pass' : name_parts.length >= 2 ? name_parts[1] : ''});
		$("#chatbox").disabled = true;
		write("tab-main").system("Connecting...");
		
	} else if (hub_state == 2) {
		if (pm_target !== false) {
			sock.emit('priv', {'user': pm_target, 'message': str});
			writerFor(pm_target).pub(hub_last_nick, str );
		} else {
			sock.emit('pub', {'message' : str});
		}
		
	} else {
		write("tab-main").system("Invalid internal state.");
	}
	
	$("#chatbox").value = '';
};

/* tabs */

var tab_set = function(tab) {
	var tabs = $(".tabpane");
	for (var i in tabs) {
		try {
			tabs[i].style.display = tabs[i].id == tab ? 'block' : 'none';
		} catch (e) {};
	}
	var tabitems = $(".tabitem");
	for (var i in tabitems) {
		try {
			tabitems[i].className = "tabitem" + (tabitems[i].getAttribute('data-tab') == tab ? ' selected' : '');
		} catch (e) {};
	}
	
	pm_target = false;
	for (var i in pm_tabs) {
		if (pm_tabs[i] == tab) {
			pm_target = i;
			break;
		}
	}
	
	updateTitle();
	
	write(tab).scroll();
	$("#chatbox").focus();
	last_tab = tab;
}

var tab_new = function(id, name) {
	$("#bar").innerHTML +=
		' <div class="tabitem" data-tab="tab-ext-'+id+'" id="tabitem-tab-ext-'+id+'">'+
			'<span class="tab-label">'+
				hesc(name)+
			'</span> '+
			'<a class="tab-closer" data-tab="tab-ext-'+id+'">&times;</a>'+
		'</div> '
	;
	$("#extratabs").innerHTML +=
		'<div class="tabpane content placement-mid" id="tab-ext-'+id+'" style="display:none;">'+
			'<div class="content-inner" id="inner-tab-ext-'+id+'"></div>'+
		'</div>'
	;
	tab_addHandlers();
	return "tab-ext-"+id;
}

var tab_free = function(id) {
	if (id == "tab-main") return;

	// remove tab item and body
	var el = $("#tabitem-"+id);
	el.parentNode.removeChild(el);
	
	var el = $("#"+id);
	el.parentNode.removeChild(el);
	
	// clear from PM tabs
	for (var i in pm_tabs) {
		if (pm_tabs[i] == id) {
			pm_tabs[i] = false;
		}
	}
	
	// maybe clear the 'new pm' warning
	updateTitle();
	
	// don't leave us with no tab displayed
	if (last_tab == id) {
		// but some event resets the tab again... hack around it until we realise the true cause at 1AM sometime
		setTimeout(function() {
			tab_set("tab-main");
		}, 1);
	}
}

var tab_addHandlers = function() {
	var tabitems = $(".tabitem");
	for (var i in tabitems) {
		if (! tabitems[i]) continue;
		
		tabitems[i].onclick = function() {
			tab_set( this.getAttribute('data-tab') );
		}
	}
	
	var tabclosers = $(".tab-closer");
	for (var i in tabclosers) {
		if (! tabclosers[i]) continue;
		
		tabclosers[i].onclick = function() {
			tab_free( this.getAttribute('data-tab') );
		}
	}
}

/* */

var writerFor = function(username) {
	var tabid = "";
	if (! username in pm_tabs || ! pm_tabs[username]) {
		tabid = tab_new(next_tabid++, username);
		pm_tabs[username] = tabid;
	} else {
		tabid = pm_tabs[username];
	}
	
	return write(tabid);
}

/* */ 

function updateTitle() {
	document.title = 
		($(".unread").length ? "[NEW PM] " : "") +
		"("+userlist.count()+") "+
		hub_hubname;
}

var sock = {};
var hub_state     = 0; // [disconnected, sent-nick, connected]
var hub_last_nick = '';
var hub_hubname   = 'DCWebUI';

var pm_tabs = {}; // nick => tabid
var next_tabid = 1;
var pm_target = false;

var last_tab = "tab-main";

window.onload = function() {
	write("tab-main").system("Communicating with server...");
	
	document.title = DCWEBUI_CONF.title;
	
	// HTML event handlers
	
	$("#form-none").onsubmit = function() {
		submit();
		return false; // don't submit form
	};
	
	tab_addHandlers();
	
	// Socket event handlers
	
	sock = io.connect('http://'+DCWEBUI_CONF.extern);
	sock.on('cls', function() {
		write("tab-main").cls();
		userlist.clear();
		hub_state = 0;
	});
	sock.on('hubname', function(s) {
		write("tab-main").system("Now talking on "+s);
		hub_hubname = s;
		updateTitle();
	});
	sock.on('sys', function(data) {
		write("tab-main").system(data);
	});
	sock.on('raw', function(data) {
		write("tab-main").raw(data);
	});
	sock.on('pub', function(data) {
		write("tab-main").pub( data.user, data.message);
	});
	sock.on('priv', function(data) {
		writerFor(data.user).pub( data.user, data.message);
		if (last_tab != pm_tabs[data.user] &&
			($("#tabitem-"+pm_tabs[data.user]).className.indexOf('unread') == -1)
		) {
			$("#tabitem-"+pm_tabs[data.user]).className += " unread";
			updateTitle();
		}
	});
	sock.on('hello', function() {
		hub_state = 2;
		$("#chatbox").disabled = false;
	});
	sock.on('part', function(u) {
		userlist.del(u.user);
		updateTitle();
	});
	sock.on('join', function(u) {
		userlist.add(u.user);
		updateTitle();
	});
	sock.on('close', function() {
		hub_state = 0;
		userlist.clear();
		write("tab-main").system("Connection closed by remote host.");
	});
	sock.on('disconnect', function() {
		hub_state = 0;
		userlist.clear();
		write("tab-main").system("Lost connection to the server.");
	});
};
